/*
 *  wavin.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

unsigned long	wavSampRate = 44100;
int				wavChannels = 2;
int				wavBits = 16;
int				wavSampleBytes = 2; // bits / 8bit
int				wavBytesPerSec=0;
int				totalbytes = 0;
unsigned long   datahead = 0,datapos = 0;
int				length_sec;

FILE *wavfile = NULL;
char myFileName[0x600];
char header[0x200];

inline unsigned long readLongLE(void *buf)
{
	unsigned char *p = buf;
	return p[0]|((unsigned long)p[1]<<8)|((unsigned long)p[2]<<16)|((unsigned long)p[3]<<24);
}
inline unsigned short readWordLE(void *buf)
{
	unsigned char *p = buf;
	return p[0]|((unsigned short)p[1]<<8);
}

inline short readSWordLE(void *buf)
{
	unsigned char *p = buf;
	return p[0]|((unsigned short)p[1]<<8);
}


inline void writeLongLE(unsigned char *p,unsigned long value)
{
	p[0]=value&0xff;
	p[1]=(value>>8)&0xff;
	p[2]=(value>>16)&0xff;
	p[3]=(value>>24)&0xff;
}
inline void writeWordLE(void *buf,unsigned short value)
{
	unsigned char *p = buf;
	p[0]=value&0xff;
	p[1]=(value>>8)&0xff;
}

void writeID(unsigned char *p,unsigned char *str)
{
	memcpy(p,str,strlen(str));
}

int GetWAVLength(void)
{
	return length_sec;
}

int GetWAVFreq(void)
{
	return wavSampRate;
}
int GetWAVChannels(void)
{
	return wavChannels;
}
int GetWAVBits(void)
{
	return wavBits;
}

unsigned long GetWAVTotalBytes(void)
{
	return totalbytes;
}

void SetWAVSeek(int sec)
{
	int i,j;
	i = (wavSampleBytes*wavChannels);
	j = ((long)(wavBytesPerSec * sec)/i)*i;
	fseek(wavfile,datahead + j,SEEK_SET);
	datapos = j;
}

int GetWAVPos(void)
{
	return datapos/wavBytesPerSec;
}

int readFromWAV(unsigned char *buf,long len)
{
	if (!wavfile) return -1; 
	fread(buf,len,1,wavfile);
	return 0;
}


int readWAVheader(FILE *fp)
{
	unsigned char buf[0x40];
	int i,j,chnk;
	
	fread(buf,4,1,fp);
	if (memcmp(buf,"RIFF",4)) return -1;
	fread(buf,4,1,fp); // size of this file

	fread(buf,4,1,fp);
	if (memcmp(buf,"WAVE",4)) return -1;
	fread(buf,4,1,fp);
	if (memcmp(buf,"fmt",3)) return -1;
	fread(buf,4,1,fp);
	chnk = readLongLE(buf);
	chnk+= ftell(fp);
	
//	if (readLongLE(buf)!=0x12) return -1; // chunk size check
	fread(buf,2,1,fp);
	if (readWordLE(buf)!=0x01) return -1; // PCM format check
	fread(buf,2,1,fp);
	wavChannels = readWordLE(buf);
	fread(buf,4,1,fp);
	wavSampRate = readLongLE(buf);
	fread(buf,4,1,fp);
	fread(buf,2,1,fp);
	fread(buf,2,1,fp);
	wavBits = readWordLE(buf);
	
	fseek(fp,chnk,SEEK_SET);
	
	wavSampleBytes = wavBits / 8;
	wavBytesPerSec = (wavSampleBytes * wavChannels * wavSampRate);
	
	j=0;
	do {
		fread(buf,4,1,fp);
		if (!memcmp(buf,"data",4)) {
			fread(buf,4,1,fp);
			totalbytes=readLongLE(buf);

			datahead = ftell(fp);
			datapos = 0;

			length_sec = totalbytes/wavBytesPerSec;
			return 0;
		}
		fread(buf,4,1,fp);
		if ((i=readLongLE(buf)) >0) fseek(fp,i,SEEK_CUR);
		j++;
		
	}while(!feof(fp) && j < 5);
	return -1;
	
}

int InitWAV(const char *fname)
{
	// make initalization code in here which kind of memory allocation or any
	strcpy(myFileName,fname);
	if (!(wavfile = fopen(myFileName,"rb")))
	{
			return -1;
	}
	return readWAVheader(wavfile);
}

void CloseWAV ()
{
	if (wavfile)
	{
		fclose(wavfile);
		wavfile = NULL;
	}
	//	make releasing code in here which kind of releasing memory , close opened file or any.
}

